// Calculator.cpp: implementation of the Calculator class.
//
//////////////////////////////////////////////////////////////////////

#include "Calculator.h"

#include <axis/common/GDefine.h>
#include <axis/common/IWrapperSoapDeSerializer.h>
#include <axis/common/IWrapperSoapSerializer.h>
#include <axis/common/IParam.h>

extern int Axis_DeSerialize_Point(Point* param, IWrapperSoapDeSerializer *pDZ);
extern void* Axis_Create_Point(bool bArray = false, int nSize=0);
extern void Axis_Delete_Point(Point* param, bool bArray = false, int nSize=0);
extern int Axis_Serialize_Point(Point* param, IWrapperSoapSerializer& pSZ, bool bArray = false);
extern int Axis_GetSize_Point(Point* param);

//Parameters and wrapper methods to manipulate Point
static const AxisChar* Axis_URI_Point = "http://www.opensource.lk/Point";
static const AxisChar* Axis_TypeName_Point = "Point";

extern int Axis_DeSerialize_Polygon(Polygon* param, IWrapperSoapDeSerializer *pDZ);
extern void* Axis_Create_Polygon(bool bArray = false, int nSize=0);
extern void Axis_Delete_Polygon(Polygon* param, bool bArray = false, int nSize=0);
extern int Axis_Serialize_Polygon(Polygon* param, IWrapperSoapSerializer& pSZ, bool bArray = false);
extern int Axis_GetSize_Polygon(Polygon* param);

//Parameters and wrapper methods to manipulate Polygon
static const AxisChar* Axis_URI_Polygon = "http://www.opensource.lk/Polygon";
static const AxisChar* Axis_TypeName_Polygon = "Polygon";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Calculator::Calculator()
{
	m_pCall = new Call();
	/* The protocol can be taken from the binding section in the WSDL */
	m_pCall->SetProtocol(APTHTTP);
	/* The value for SOAPAction is specified in the binding section of the WSDL */
	m_pCall->SetHeader("SOAPAction", "Calculator");
	/* End point URI too can be taken from WSDL */

	m_pCall->SetEndpointURI("http://localhost/axis/Calculator");


}

Calculator::~Calculator()
{
	delete m_pCall;
}

int Calculator::Add(int a, int b)
{
	int nStatus;
	int ret = 0;
	/* Following will establish the connections with the server too */
	if (AXIS_SUCCESS != m_pCall->Initialize()) 
		return ret;
	m_pCall->SetSOAPVersion(SOAP_VER_1_1);
	/* This should be done before adding parameters or return type */
	m_pCall->SetOperation("Add", "http://www.opensource.lk/");
	m_pCall->AddParameter(a, "a", XSD_INT);
	m_pCall->AddParameter(b, "b", XSD_INT);
	m_pCall->SetReturnType(XSD_INT);
	nStatus = m_pCall->Invoke();
	if (AXIS_SUCCESS == nStatus)
	{
        printf("if (AXIS_SUCCESS == nStatus)");
		m_pCall->GetResult()->GetInt(&ret);
#ifdef _DEBUG
		printf("%s%d", "Got The result", ret);
#endif
	}
	/* Following will close the connection with the server too */
	m_pCall->UnInitialize();
	return ret;
}

Point* Calculator::AddPoint(Point* p1, Point* p2)
{
	int nStatus;
	Point* pReturn = NULL;
	if (AXIS_SUCCESS != m_pCall->Initialize()) return NULL;
	m_pCall->SetSOAPVersion(SOAP_VER_1_1);
	m_pCall->SetOperation("AddPoint", "http://www.opensource.lk/");//this should be done before adding parameters or return type
	m_pCall->AddParameter(p1, (void*) Axis_Serialize_Point, (void*) Axis_Delete_Point, "p1");
	m_pCall->AddParameter(p2, (void*) Axis_Serialize_Point, (void*) Axis_Delete_Point, "p2");
	m_pCall->SetReturnType((void*) Axis_DeSerialize_Point, (void*) Axis_Create_Point, (void*) Axis_Delete_Point, Axis_TypeName_Point, Axis_URI_Point);
	nStatus = m_pCall->Invoke();
	if (AXIS_SUCCESS == nStatus)
	{
		m_pCall->GetResult((void**)&pReturn);
	}
	m_pCall->UnInitialize();
	return pReturn;
}

PointArray Calculator::EchoPointArray(PointArray Array)
{
	int nStatus;
	PointArray RetArray = {NULL, 0};
	if (AXIS_SUCCESS != m_pCall->Initialize()) return RetArray;
	m_pCall->SetSOAPVersion(SOAP_VER_1_1);
	m_pCall->SetOperation("EchoPointArray", "http://www.opensource.lk/");//this should be done before adding parameters or return type
	m_pCall->AddParameter((Axis_Array*)(&Array), (void*) Axis_Serialize_Point, (void*) Axis_Delete_Point, (void*) Axis_GetSize_Point, Axis_TypeName_Point, Axis_URI_Point, "PointArray");
	m_pCall->SetReturnType((Axis_Array*)(&RetArray), (void*) Axis_DeSerialize_Point, (void*) Axis_Create_Point, (void*) Axis_Delete_Point, (void*) Axis_GetSize_Point, Axis_TypeName_Point, Axis_URI_Point);
	nStatus = m_pCall->Invoke();
	if (AXIS_SUCCESS != nStatus)
	{
		delete RetArray.m_Array;
		RetArray.m_Array = NULL;
		RetArray.m_Size = 0;
	}
	m_pCall->UnInitialize();
	return RetArray;
}

Polygon* Calculator::EchoPolygon(Polygon* p)
{
	int nStatus;
	Polygon* pReturn = NULL;
	if (AXIS_SUCCESS != m_pCall->Initialize()) return NULL;
	m_pCall->SetSOAPVersion(SOAP_VER_1_1);
	m_pCall->SetOperation("EchoPolygon", "http://www.opensource.lk/");//this should be done before adding parameters or return type
	m_pCall->AddParameter(p, (void*) Axis_Serialize_Polygon, (void*) Axis_Delete_Polygon, "p");
	m_pCall->SetReturnType((void*) Axis_DeSerialize_Polygon, (void*) Axis_Create_Polygon, (void*) Axis_Delete_Polygon, Axis_TypeName_Polygon, Axis_URI_Polygon);
	nStatus = m_pCall->Invoke();
	if (AXIS_SUCCESS == nStatus)
	{
		m_pCall->GetResult((void**)&pReturn);
	}
	m_pCall->UnInitialize();
	return pReturn;
}
