/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 *
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

// webservicewrapper.cpp: implementation of the webservicewrapper class.
//
//////////////////////////////////////////////////////////////////////

#include "WebServiceWrapper.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

static const AxisChar* Axis_URI_Point = "http://www.opensource.lk/Point";
static const AxisChar* Axis_TypeName_Point = "Point";

int Axis_DeSerialize_Point(Point* p, IWrapperSoapDeSerializer *pDZ)
{
	p->x = pDZ->GetInt();
	p->y = pDZ->GetInt();
	return AXIS_SUCCESS;
}

void* Axis_Create_Point(bool bArray = false, int nSize=0)
{
	if (bArray && (nSize > 0))
		return new Point[nSize];
	else
		return new Point;
}

void Axis_Delete_Point(Point* p, bool bArray = false, int nSize=0)
{
	if (bArray)
		delete [] p;
	else
		delete p;
}

int Axis_Serialize_Point(Point* p, IWrapperSoapSerializer& pSZ, bool bArray = false)
{
	if (bArray)
	{
		pSZ << "<" << Axis_TypeName_Point << ">";
	}
	else
	{
		AxisString sPrefix = pSZ.getNewNamespacePrefix();
		pSZ << "<" << Axis_TypeName_Point << " xsi:type=\"" << sPrefix.c_str() <<":"
			<< Axis_TypeName_Point << "\" xmlns:" << sPrefix.c_str() << "=\""
			<< Axis_URI_Point << "\">";
	}
	pSZ << pSZ.SerializeBasicType("x", p->x, XSD_INT);
	pSZ << pSZ.SerializeBasicType("y", p->y, XSD_INT);
	pSZ << "</" << Axis_TypeName_Point << ">";
	return AXIS_SUCCESS;
}

int Axis_GetSize_Point(Point* p)
{
	return sizeof(Point);
}

webservicewrapper::webservicewrapper()
{
	pWs = new WebService(); //Create actual web service object here
}

webservicewrapper::~webservicewrapper()
{
	delete pWs;
}
// Implementation of WrapperClassHandler interfaces.
int webservicewrapper::Invoke(IMessageData* mc)
{
	IWrapperSoapDeSerializer* pIWrapperSoapDeSerializer;
	mc->getSoapDeSerializer(&pIWrapperSoapDeSerializer);
	const AxisChar* method = pIWrapperSoapDeSerializer->GetMethodName();
	if (0 == strcmp(method, "Echo"))
		return Echo(mc);
	else if (0 == strcmp(method, "Add"))
    {
		return Add(mc);
    }
	else if (0 == strcmp(method, "EchoIntArray"))
		return EchoIntArray(mc);
	else if (0 == strcmp(method, "AddPoint"))
		return AddPoint(mc);
	else if (0 == strcmp(method, "EchoPointArray"))
		return EchoPointArray(mc);
	return AXIS_FAIL;
}
int webservicewrapper::Init()
{
	//Do any Initialization here
	return 0;
}
int webservicewrapper::Fini()
{
	//Do any finalization here
	return 0;
}

/*int webservicewrapper::SetResponseMethod(IMessageData* mc, const AxisChar* name)
{
	IWrapperSoapSerializer* pIWSz = NULL;
	mc->getSoapSerializer(&pIWSz);
	if (!pIWSz) return AXIS_FAIL;
	ISoapMethod* pMethod= pIWSz->createSoapMethod();
	pMethod->setLocalName(name);
	pMethod->setPrefix("m");
	pMethod->setUri("http://www.opensource.lk");
	return AXIS_SUCCESS;
} */
// Methods that wraps actual web methods

int webservicewrapper::Echo(IMessageData* mc)
{
	//SetResponseMethod(mc, "EchoResponse");
	IWrapperSoapDeSerializer* pIWrapperSoapDeSerializer;
	mc->getSoapDeSerializer(&pIWrapperSoapDeSerializer);
	IWrapperSoapSerializer* pIWSz;
	mc->getSoapSerializer(&pIWSz);
	pIWSz->createSoapMethod("EchoResponse", pIWSz->getNewNamespacePrefix(), "http://www.opensource.lk/WebService");
	//IParam *param1 = pIWrapperSoapDeSerializer->GetParam();
	//We know that parameter 1 should be an string
	AxisString str = pIWrapperSoapDeSerializer->GetString();
	//Call actual web service method with appropriate parameters
	AxisString strRet = pWs->Echo(str);
	//IParam* pRetParam = pIWSz->AddOutputParam(strRet.c_str());
	//pRetParam->SetName("EchoReturn");
	//return AXIS_SUCCESS;
    return pIWSz->AddOutputParam("EchoReturn", strRet, XSD_STRING);
}

int webservicewrapper::Add(IMessageData* mc)
{
	//SetResponseMethod(mc, "AddResponse");
	IWrapperSoapDeSerializer* pIWrapperSoapDeSerializer = NULL;
	mc->getSoapDeSerializer(&pIWrapperSoapDeSerializer);
	IWrapperSoapSerializer* pIWSz = NULL;
	mc->getSoapSerializer(&pIWSz);
	pIWSz->createSoapMethod("AddResponse", pIWSz->getNewNamespacePrefix(), "http://www.opensource.lk/WebService");
	//IParam *param1 = pIWrapperSoapDeSerializer->GetParam();
	//IParam *param2 = pIWrapperSoapDeSerializer->GetParam();
	//Convert parameters to actual types
	int i1 = pIWrapperSoapDeSerializer->GetInt();
	int i2 = pIWrapperSoapDeSerializer->GetInt();
	//Call actual web service method with appropriate parameters
	int ret = pWs->Add(i1, i2);
	//IParam* pRetParam = pIWSz->AddOutputParam(ret);
	//pRetParam->SetName("AddReturn");
	return pIWSz->AddOutputParam("AddReturn", ret, XSD_INT);
}

void webservicewrapper::OnFault(IMessageData* mc)
{
}
/*
int webservicewrapper::EchoIntArray(IMessageData *mc)
{
	SetResponseMethod(mc, "EchoIntArrayResponse");
	IWrapperSoapDeSerializer* pIWrapperSoapDeSerializer;
	mc->getSoapDeSerializer(&pIWrapperSoapDeSerializer);
	IParam *param1 = pIWrapperSoapDeSerializer->GetParam();
	int size = param1->GetArraySize(); //gets full array size
	int * arr = new int[size];
	param1->SetArrayElements((void*)arr);
	pIWrapperSoapDeSerializer->Deserialize(param1,0);
	int retarrsize = 0;
	int * retarr;
	retarr = pWs->EchoIntArray(size,arr,&retarrsize);
	IWrapperSoapSerializer* pIWSz;
	mc->getSoapSerializer(&pIWSz);
	IArrayBean* pAb = pIWSz->makeArrayBean(XSD_INT, (void*)retarr);
	pAb->AddDimension(retarrsize);
	pAb->SetItemName("item");
	uParamValue value;
	value.pIArray = pAb;
//	IParam* pRetParam = pIWSz->AddOutputParam(XSD_ARRAY, value);
//	pRetParam->SetName("EchoIntArrayReturn");
	return AXIS_SUCCESS;
}*/

int webservicewrapper::EchoIntArray(IMessageData* mc)
{
	IWrapperSoapSerializer* pIWSSZ = NULL;
	mc->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	pIWSSZ->createSoapMethod("EchoIntArrayResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/WebService");

	//we expect a IntArray now
	Axis_Array inArray = pIWSDZ->GetArray(XSD_INT);

	IntArray outArray = pWs->EchoIntArray((IntArray&)inArray);

	return pIWSSZ->AddOutputParam("EchoIntArrayReturn", (Axis_Array*)(&outArray),
		(void*) Axis_Serialize_Point, (void*) Axis_Delete_Point, (void*) Axis_GetSize_Point,
		Axis_TypeName_Point, Axis_URI_Point);
}

int webservicewrapper::AddPoint(IMessageData *mc)
{
	IWrapperSoapSerializer* pIWSSZ = NULL;
	mc->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	//SetResponseMethod(mc, "AddPointResponse");

	//IParam *param0 = pIWSDZ->GetParam();
	//Point* v0 = new Point();

	//param0->SetUserType(v0, (AXIS_DESERIALIZE_FUNCT)Axis_DeSerialize_Point, (AXIS_OBJECT_DELETE_FUNCT)Axis_Delete_Point);
	//pIWSDZ->Deserialize(param0,0);

	//IParam *param1 = pIWSDZ->GetParam();
	//Point* v1 = new Point();

	//param1->SetUserType(v1, (AXIS_DESERIALIZE_FUNCT)Axis_DeSerialize_Point, (AXIS_OBJECT_DELETE_FUNCT)Axis_Delete_Point);
	//pIWSDZ->Deserialize(param1,0);

    pIWSSZ->createSoapMethod("AddPointResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/WebService");

	Point* v0 = (Point*)pIWSDZ->GetObject((void*) (AXIS_DESERIALIZE_FUNCT)Axis_DeSerialize_Point,
		(void*) (AXIS_OBJECT_CREATE_FUNCT)Axis_Create_Point, (void*) (AXIS_OBJECT_DELETE_FUNCT)Axis_Delete_Point,
		Axis_TypeName_Point, Axis_URI_Point);

	Point* v1 = (Point*)pIWSDZ->GetObject((void*) (AXIS_DESERIALIZE_FUNCT)Axis_DeSerialize_Point,
		(void*) (AXIS_OBJECT_CREATE_FUNCT)Axis_Create_Point, (void*) (AXIS_OBJECT_DELETE_FUNCT)Axis_Delete_Point,
		Axis_TypeName_Point, Axis_URI_Point);
    
	//Call actual web service method with appropriate parameters
	Point* ret = pWs->AddPoint(v0, v1);

	//IParam* pRetParam = pIWSSZ->AddOutputParam(ret, (void*) Axis_Serialize_Point, (void*) Axis_Delete_Point);
	//pRetParam->SetName("AddPointReturn");
	//return AXIS_SUCCESS;
    return pIWSSZ->AddOutputParam("AddPointReturn", ret, (void*)Axis_Serialize_Point, (void*)Axis_Delete_Point);
}

/*int webservicewrapper::EchoPointArray(IMessageData *mc)
{
	SetResponseMethod(mc, "EchoPointArrayResponse");
	IWrapperSoapDeSerializer* pIWrapperSoapDeSerializer;
	mc->getSoapDeSerializer(&pIWrapperSoapDeSerializer);
	IParam *param1 = pIWrapperSoapDeSerializer->GetParam();
	int size = param1->GetArraySize(); //gets full array size
	Point* arr = new Point[size];
	param1->SetArrayElements((void*)arr, (AXIS_DESERIALIZE_FUNCT)Axis_DeSerialize_Point, (AXIS_OBJECT_DELETE_FUNCT)Axis_Delete_Point, (AXIS_OBJECT_SIZE_FUNCT)Axis_GetSize_Point);
	pIWrapperSoapDeSerializer->Deserialize(param1,0);
	Point* pRet = pWs->EchoPointArray(size, arr,&size);
	IWrapperSoapSerializer* pIWSz;
	mc->getSoapSerializer(&pIWSz);
	IArrayBean* pAb = pIWSz->makeArrayBean((void*)pRet, (void*) Axis_Serialize_Point, (void*) Axis_Delete_Point, (void*) Axis_GetSize_Point);
	pAb->AddDimension(size);
	pAb->SetItemName("item");
	pAb->SetTypeName("Point");
	pAb->SetUri("http://www.opensource.lk/Point");
	IParam* pRetParam = pIWSz->AddOutputParam(pAb);
	pRetParam->SetName("EchoPointArrayReturn");
	return AXIS_SUCCESS;
}*/

int webservicewrapper::EchoPointArray(IMessageData* mc)
{
	IWrapperSoapSerializer* pIWSSZ = NULL;
	mc->getSoapSerializer(&pIWSSZ);
	if (!pIWSSZ) return AXIS_FAIL;
	IWrapperSoapDeSerializer* pIWSDZ = NULL;
	mc->getSoapDeSerializer(&pIWSDZ);
	if (!pIWSDZ) return AXIS_FAIL;
	pIWSSZ->createSoapMethod("EchoPointArrayResponse", pIWSSZ->getNewNamespacePrefix(), "http://www.opensource.lk/WebService");

	//we expect a PointArray now
	Axis_Array inArray = pIWSDZ->GetArray((void*) (AXIS_DESERIALIZE_FUNCT)Axis_DeSerialize_Point,
		(void*) (AXIS_OBJECT_CREATE_FUNCT)Axis_Create_Point, (void*) (AXIS_OBJECT_DELETE_FUNCT)Axis_Delete_Point,
		(void*) (AXIS_OBJECT_SIZE_FUNCT)Axis_GetSize_Point, Axis_TypeName_Point, Axis_URI_Point);

	PointArray outArray = pWs->EchoPointArray((PointArray&)inArray);

	return pIWSSZ->AddOutputParam("EchoPointArrayReturn", (Axis_Array*)(&outArray),
		(void*) Axis_Serialize_Point, (void*) Axis_Delete_Point, (void*) Axis_GetSize_Point,
		Axis_TypeName_Point, Axis_URI_Point);
}

